#include "mpu6050.h"

#define MPU6050_ADDRESS     0xD0

/**
  **************************************************************************
  ** @name          : MPU6050_Init
  **************************************************************************
**/
void MPU6050_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure={0};
    I2C_InitTypeDef  I2C_InitStructure={0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;     //ÿ©ӲI2CЭҪӲ
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);


    I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C_InitStructure.I2C_ClockSpeed = 50000;
    I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
    I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStructure.I2C_OwnAddress1 = 0x00;
    I2C_Init(I2C2, &I2C_InitStructure);

    I2C_Cmd(I2C2, ENABLE);

    MPU6050_WriteReg(MPU6050_PWR_MGMT_1, 0x01);
    MPU6050_WriteReg(MPU6050_PWR_MGMT_2, 0x00);
    MPU6050_WriteReg(MPU6050_SMPLRT_DIV, 0x09);
    MPU6050_WriteReg(MPU6050_CONFIG, 0x06);
    MPU6050_WriteReg(MPU6050_GYRO_CONFIG, 0x18);
    MPU6050_WriteReg(MPU6050_ACCEL_CONFIG, 0x18);
}


/**
  **************************************************************************
  ** @name          : MPU6050_WaitEvent
  ** @brief         : ֹѭʱ˳
  **************************************************************************
**/
void MPU6050_WaitEvent(I2C_TypeDef* I2Cx, uint32_t I2C_EVENT)
{
    uint32_t Timeout=10000;
    while (I2C_CheckEvent(I2Cx, I2C_EVENT) != SUCCESS)
    {
        Timeout --;
        if (Timeout == 0)
        {
            break;//ɼܣӡ
        }
    }
}


/**
  **************************************************************************
  ** @name          : MPU6050_WriteReg
  ** @brief         : ַָдĴ
  ** @attention     : ϸʽַָ
  **************************************************************************
**/
void MPU6050_WriteReg(uint8_t RegAddress, uint8_t Data)//дĴ
{

    I2C_GenerateSTART(I2C2, ENABLE);                        //ʼ(дͽ˳ʽĺ)
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT);  //ȴEV5¼ȷ溯ִеλ

    I2C_Send7bitAddress(I2C2, MPU6050_ADDRESS, I2C_Direction_Transmitter); //ʹӻַ
    /***ͺͽԴӦ***/
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED);   //ȴEV6¼

    I2C_SendData(I2C2, RegAddress);                             //дDRݣԶתλĴ
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTING);//ȴEV8¼

    I2C_SendData(I2C2, Data);                                   //һֽData
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED); //ȴEV8_2¼λˣٽˣֽѾ

    I2C_GenerateSTOP(I2C2, ENABLE);     //ֹʱ
}


/**
  **************************************************************************
  ** @name          : MPU6050_ReadReg
  ** @brief         : ַָĴ
  ** @attention     : ϸʽַָ
  **************************************************************************
**/
uint8_t MPU6050_ReadReg(uint8_t RegAddress)//Ĵ
{
    uint8_t Data;

    I2C_GenerateSTART(I2C2, ENABLE);  //ʼ
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT);//ȴEV5¼

    I2C_Send7bitAddress(I2C2, MPU6050_ADDRESS, I2C_Direction_Transmitter); //ʹӻַ
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED);    //ȴEV6¼

    I2C_SendData(I2C2, RegAddress);//
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED);//ȴEV8¼

    I2C_GenerateSTART(I2C2, ENABLE);//ظʼ
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT);//ȴEV5¼

    I2C_Send7bitAddress(I2C2, MPU6050_ADDRESS, I2C_Direction_Receiver); //ʹӻַ
    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED); //ȴEV6¼
    /******************************************************************************************
           һֽ֮ǰǰACKSTOP
    ******************************************************************************************/
    I2C_AcknowledgeConfig(I2C2, DISABLE);//ǰACK0Ӧ
    I2C_GenerateSTOP(I2C2, ENABLE);//ǰֹSTOP=1

    MPU6050_WaitEvent(I2C2, I2C_EVENT_MASTER_BYTE_RECEIVED);//ȴEV7¼һѾ浽DRУ
    Data = I2C_ReceiveData(I2C2);//ȡDRݣ浽Data

    I2C_AcknowledgeConfig(I2C2, ENABLE);//ACKظĬϵ1Ϊ˷ַָնֽڣ

    return Data;
}


/**
  **************************************************************************
  ** @name          : MPU6050_GetID
  ** @brief         : ȡMPU6050ID
  **************************************************************************
**/
uint8_t MPU6050_GetID(void)
{
    return MPU6050_ReadReg(MPU6050_WHO_AM_I);
}

/**
  **************************************************************************
  ** @name          : MPU6050_GetData
  ** @brief         : ȡMPU6050ĽٶȺ
  ** @param         : int16_t *AccX, int16_t *AccY, int16_t *AccZ,
                      int16_t *GyroX, int16_t *GyroY, int16_t *GyroZ
  **************************************************************************
**/
void MPU6050_GetData(int16_t *AccX, int16_t *AccY, int16_t *AccZ,
    int16_t *GyroX, int16_t *GyroY, int16_t *GyroZ)
{
    uint8_t DataH, DataL;

    DataH = MPU6050_ReadReg(MPU6050_ACCEL_XOUT_H);
    DataL = MPU6050_ReadReg(MPU6050_ACCEL_XOUT_L);
    *AccX = (DataH << 8) | DataL;

    DataH = MPU6050_ReadReg(MPU6050_ACCEL_YOUT_H);
    DataL = MPU6050_ReadReg(MPU6050_ACCEL_YOUT_L);
    *AccY = (DataH << 8) | DataL;

    DataH = MPU6050_ReadReg(MPU6050_ACCEL_ZOUT_H);
    DataL = MPU6050_ReadReg(MPU6050_ACCEL_ZOUT_L);
    *AccZ = (DataH << 8) | DataL;

    DataH = MPU6050_ReadReg(MPU6050_GYRO_XOUT_H);
    DataL = MPU6050_ReadReg(MPU6050_GYRO_XOUT_L);
    *GyroX = (DataH << 8) | DataL;

    DataH = MPU6050_ReadReg(MPU6050_GYRO_YOUT_H);
    DataL = MPU6050_ReadReg(MPU6050_GYRO_YOUT_L);
    *GyroY = (DataH << 8) | DataL;

    DataH = MPU6050_ReadReg(MPU6050_GYRO_ZOUT_H);
    DataL = MPU6050_ReadReg(MPU6050_GYRO_ZOUT_L);
    *GyroZ = (DataH << 8) | DataL;

    printf("ACC:  X=%d   Y=%d   Z=%d  \n",*AccX,*AccY,*AccZ);
    printf("GYRO:  X=%d   Y=%d   Z=%d  \n",*GyroX,*GyroY,*GyroZ);

}

